@@ -10,7 +10,7 @@ SRE.predict <- function(...) {
 #' @export
 setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs = FALSE,
                                                pred_time = NULL, covariances = FALSE, 
-                                               n_MC = 400, type = "mean", k = NULL, 
+                                               nsim = 400, type = "mean", k = NULL, 
                                                percentiles = c(5, 95), 
                                                kriging = "simple") {
   
@@ -24,7 +24,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   ## The user can either provide k in object@BAUs$k_BAU, or in the predict call.
   ## This is so that the user can change k without having to call SRE() and SRE.fit() again.
   ## The k supplied in predict() will take precedence over the k stored in object@BAUs$k.
-
+  
   k_BAU <- object@BAUs$k_BAU # size parameter at the BAU level (could be NULL)
   N <- length(object@BAUs)   # number of BAUs
   
@@ -32,7 +32,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
     if (is.null(k)) {
       if(is.null(k_BAU)) {
         k <- rep(1, N)
-        cat("Assuming the size parameter, k, is equal to 1 for all prediction locations: If you want to change this, use the argument 'k' in predict.\n")
+        cat("Assuming the size parameter, k, is equal to 1 for all prediction locations: If you want to change this, use the argument 'k' in predict().\n")
       } else {
         if (any(is.na(k_BAU))) {
           
@@ -87,7 +87,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   ## compute the full covariance matrix. Note this by setting
   ## predict_BAUs to be FALSE
   if (!is.null(newdata)) {
-    tmp <- as(CP, "dgTMatrix")
+    tmp <- .as(CP, "dgTMatrix")
     if (!all(table(tmp@i) == 1))
       predict_BAUs <- FALSE
   }
@@ -103,12 +103,12 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
                                  predict_BAUs = predict_BAUs) # Are we predicting at BAUs?                   
     
   } else if (object@method == "TMB") {
-    pred_locs <- .SRE.predict_TMB(M = object,               # Fitted SRE model
+    pred_locs <- .SRE.predict_TMB(object = object,               # Fitted SRE model
                                   newdata = newdata,           # Prediction polygons
                                   CP = CP,                     # Polygon prediction matrix
                                   predict_BAUs = predict_BAUs, # Are we predicting at BAUs?
                                   pred_time = pred_time,       # Prediction time points
-                                  n_MC = n_MC,                 # Desired number of MC simulations
+                                  nsim = nsim,                 # Desired number of MC simulations
                                   obs_fs = obs_fs,             # Case 1 or Case 2?
                                   type = type,                 # Whether we are interested in the "link" (Y-scale), "mean", "response"
                                   k = k,                       # Size parameter
@@ -230,8 +230,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   
   ## If we have to compute too many covariances then stop and give error
   if(covariances & nrow(CP) > 4000)
-    stop("Cannot compute covariances for so many prediction locations. Please reduce
-             to less than 4000")
+    stop("Cannot compute covariances for so many prediction locations. Please reduce to less than 4000")
   
   ## Get the CZ matrix
   CZ <- Sm@Cmat
@@ -242,8 +241,8 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   if(is(newdata,"Spatial")) {
     
     ## The needed BAUs are the nonzero column indices of CZ and CP
-    needed_BAUs <- union(as(CP,"dgTMatrix")@j+1,
-                         as(CZ,"dgTMatrix")@j+1)
+    needed_BAUs <- union(.as(CP,"dgTMatrix")@j+1,
+                         .as(CZ,"dgTMatrix")@j+1)
     
     ## Filter the BAUs and the matrices
     BAUs <- BAUs[needed_BAUs,]
@@ -252,14 +251,6 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
     Sm@S0 <- Sm@S0[needed_BAUs,]
   }
   
-  # Deprecated:
-  # if(is(BAUs,"ST")){
-  #     needed_BAUs <- BAUs[,pred_time]$n
-  #     BAUs <- BAUs[,pred_time]
-  #     CP <- CP[,needed_BAUs]
-  #     CZ <- CZ[,needed_BAUs]
-  # }
-  
   ## Retrieve the dependent variable name
   depname <- all.vars(Sm@f)[1]
   
@@ -274,7 +265,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   
   ## Set variables to make code more concise
   S0 <- Sm@S0
-  S0 <- as(S0,"dgCMatrix")   # ensure S0 is classified as sparse
+  S0 <- .as(S0,"dgCMatrix")   # ensure S0 is classified as sparse
   alpha <- Sm@alphahat
   K <- Sm@Khat
   sigma2fs <- Sm@sigma2fshat
@@ -292,8 +283,8 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
       Dinv <- chol2inv(Dchol)
     }
     if(sigma2fs > 0) {
-      sig2_Vfs_pred <- Diagonal(x=sigma2fs*BAUs$fs)   # fine-scale variation including estimated factor
-      Q <- solve(sig2_Vfs_pred)                       # precision of fine-scale variation
+         sig2_Vfs_pred <- Diagonal(x=sigma2fs*BAUs$fs)   # fine-scale variation including estimated factor
+         Q <- solve(sig2_Vfs_pred)                       # precision of fine-scale variation
     }
   } else  {
     stop("Prediction for other models not yet implemented")
@@ -313,7 +304,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
       tC_Ve_C <- t(CZ) %*% solve(Sm@Ve) %*% CZ +       # summary matrix
         0*.symDiagonal(ncol(CZ))              # Ensure zeros on diagonal
       Qx <- t(PI) %*% tC_Ve_C %*% PI + LAMBDAinv       # conditional precision matrix
-      chol_Qx <- cholPermute(as(Qx,"dgCMatrix"))       # permute and do Cholesky
+      chol_Qx <- cholPermute(.as(Qx,"dgCMatrix"))       # permute and do Cholesky
       ybar <- t(PI) %*%t(CZ) %*% solve(Sm@Ve) %*%      # Qx = ybar, see vignette
         (Sm@Z - CZ %*% X %*% alpha)
       x_mean <- cholsolve(Qx,ybar,perm=TRUE,           # solve Qx = ybar using permutations
@@ -419,45 +410,42 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   }
 }
 
-## #'@param CP polygon prediction matrix
-## #'@param predict_BAUs logical indicating whether or not we are predicting over the BAUs
-## #'@param pred_time vector of time indices at which prediction will be carried out. All time points are used if this option is not specified
-## #'@return A list object containing:
-## #'\describe{
-## #'  \item{newdata}{An object of class \code{newdata}, with predictions and prediction uncertainty at each prediction location of the latent \eqn{Y} process, the conditional mean of the data \eqn{\mu}, the probability of success parameter \eqn{\pi} (if applicable), and the response variable \eqn{Z}}
-## #'  \item{MC}{A list with each element being an \code{N * n_MC} matrix of Monte Carlo samples of the quantities specified by \code{type} (some combination of \eqn{Y}, \eqn{\mu}, \eqn{p} (if applicable), and \eqn{Z}) at each prediction location}
-## #'}
-.SRE.predict_TMB <- function(M, newdata, CP, predict_BAUs, pred_time, type, n_MC, 
+
+.SRE.predict_TMB <- function(object, newdata, CP, predict_BAUs, pred_time, type, nsim, 
                              obs_fs, k, percentiles, kriging) {
   
   ## Covariate design matrix at the BAU level
-  X     <- as(.extract_BAU_X_matrix(formula = M@f, BAUs = M@BAUs), "matrix")
-  obsidx <- observed_BAUs(M) # index of observed BAUs
-  p     <- length(M@alphahat)       # number of fixed regression effects
+  X     <- as(.extract_BAU_X_matrix(formula = object@f, BAUs = object@BAUs), "matrix")
+  obsidx <- observed_BAUs(object)        # index of observed BAUs
+  p     <- length(object@alphahat)       # number of fixed regression effects
   mstar <- length(obsidx)           # number of observed BAUs
-  r     <- nbasis(M)                # number of basis-function coefficients
+  r     <- nbasis(object)                # number of basis-function coefficients
+  
   ## Total number of random effects (fine-scale only included if include_fs = T)
-  s     <- r + mstar * M@include_fs 
+  s     <- r + mstar * object@include_fs 
   
-  ## Compute the Cholesky factor of the permuted precision matrix.
   ## If we are doing universal kriging, use the full joint precision 
   ## matrix of the fixed and random effects. If we are doing simple kriging, 
   ## use only the random effect block of the precision matrix.
-  if (kriging == "universal") Q_posterior <- M@Q_posterior
-  if (kriging == "simple") {
-    if (M@simple_kriging_fixed) {
-      Q_posterior <- M@Q_posterior
+  if (kriging == "universal") {
+    Q_posterior <- object@Q_posterior
+  } else if (kriging == "simple") {
+    if (object@simple_kriging_fixed) {
+      Q_posterior <- object@Q_posterior
     } else {
-      Q_posterior <- M@Q_posterior[-(1:p), -(1:p)]
+      Q_posterior <- object@Q_posterior[-(1:p), -(1:p)]
     }
   } 
+  
+  ## Compute the Cholesky factor of the permuted precision matrix.
   Q_L <- sparseinv::cholPermute(Q = Q_posterior)
   
   ## Generate Monte Carlo samples at all BAUs (or over arbitrary prediction 
   ## regions given in the argument newdata)
-  MC <- .MC_sampler(M = M, X = X, type = type, obs_fs = obs_fs, 
-                    n_MC = n_MC, k = k, Q_L = Q_L, obsidx = obsidx, 
-                    predict_BAUs = predict_BAUs, CP = CP, kriging = kriging)
+  MC <- .simulate(object = object, X = X, type = type, obs_fs = obs_fs, 
+                  nsim = nsim, k = k, Q_L = Q_L,  
+                  predict_BAUs = predict_BAUs, CP = CP, 
+                  kriging = kriging, newdata = newdata)
   
   ## We do not allow aggregation of the Y-process when predicting over arbitrary polygons
   if(!predict_BAUs) MC$Y_samples <- NULL
@@ -474,7 +462,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   
   ## If we are predicting over BAUs, newdata is NULL, so set it to the BAUs.
   ## Note that BAUs must be a Spatial*DataFrame, so coercion isn't really necessary.
-  if (predict_BAUs) newdata <- .Coerce_SpatialDataFrame(M@BAUs)
+  if (predict_BAUs) newdata <- .Coerce_SpatialDataFrame(object@BAUs)
   
   ## Now update newdata with the predictions, RMSPE, and percentiles. 
   ## (See https://datascience.stackexchange.com/a/8924 for a description of what this gsub is doing.)
@@ -484,7 +472,7 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   colnames(predictions) <- paste0("p_", QOI)
   colnames(RMSPE) <- paste0("RMSPE_", QOI)
   newdata@data <- cbind(newdata@data, predictions, RMSPE)
-
+  
   ## Percentiles 
   newdata@data <- .concat_percentiles_to_df(newdata@data, MC = MC, percentiles = percentiles)
   
@@ -499,133 +487,122 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
     }
   }
   
-  # ## It is convenient to have the spatial coordinates in the @data slot of the
-  # ## returned newdata object. Only add those coordinates not already in the data.
-  # tmp <- which(!(colnames(coordinates(newdata)) %in% names(newdata@data)))
-  # if (length(tmp))
-  #   newdata@data <- cbind(newdata@data, coordinates(newdata)[, tmp]) 
-  
   ## Return the predictions and uncertainty summary (in newdata) and the MC samples (in MC)
   return(list(newdata = newdata, MC = MC))
 }
 
 
-## #'Monte Carlo sampling of the conditional mean of the data (a function of the
-## #'latent process Y).
-## #'
-## #'Computes a Monte Carlo sample of \eqn{Y}, the conditional mean of the data
-## #'\eqn{\mu = g^-1(Y)} (which is a deterministic function of Y), the response variable \eqn{Z}, and, for response-link
-## #'combinations to which it is applicable, the probability of success parameter
-## #'p. It does so for every BAU location. 
-## #'
-## #'For negative-binomial and binomial data, the \code{BAUs} slot of the \code{SRE} object must contain a field \code{k}, which is the known constant parameter for each BAU.
-## #'For negative-binomial data, the ith element of \code{k} indicates the number of failures until the experiment is stopped at the ith BAU.
-## #'For binomial data, the ith element of \code{k} indicates the number of trials at the ith BAU.
-## #'
-## #'
-## #' @param M An object of class \code{SRE}
-## #' @param X The design matrix of the covariates at the BAU level (often simply an Nx1 column vector of 1's)
-## #' @param type A character string (possibly vector) indicating the quantities which are the focus of inference. Note: unlike in the predict() function, \emph{all} computed quantities are returned. That is, the latent \eqn{Y} process samples are always provided; If \code{"mean"} \emph{OR} \code{"response"} is in \code{type}, then the samples of \eqn{Y}, the conditonal mean \eqn{\mu}, and the probability parameter (if applicable) are provided. If \code{"response"} is in \code{type}, the response variable \eqn{Z} samples, and the samples of all other quantities are provided
-## #' @param n_MC A postive integer indicating the number of MC samples at each location
-## #' @param obs_fs flag indicating whether the fine-scale variation sits in the observation model (systematic error; indicated by \code{obs_fs = TRUE}) or in the process model (process fine-scale variation; indicated by \code{obs_fs = FALSE}, default). For non-Gaussian data models, and/or non-identity link functions, if \code{obs_fs = TRUE}, then the fine-scale variation is removed from the latent process \eqn{Y}; however, they are re-introduced for computation of the conditonal mean \eqn{\mu} and response variable \eqn{Z}
-## #' @param k vector of size parameters at each BAU (applicable only for binomial and negative-binomial data)
-## #' @param Q_L A list containing the Cholesky factor of the permuted precision matrix (stored as \code{Q$Qpermchol}) and the associated permutationmatrix (stored as \code{Q_L$P})
-## #' @param predict_BAUs logical, indicating whether we are predicting over the BAUs
-## #' @param CP the prediction incidence matrix
-## #' @param kriging whether we wish to perform "simple" or "universal" kriging
-## #' @param obsidx A vector containing the indices of observed BAUs
-## #' @return A list containing Monte Carlo samples of various quantities of interest. The list elements are (N x n_MC) matrices, whereby the ith row of each matrix corresponds to \code{n_MC} samples of the given quantity at the ith BAU. The available quantities are:
-## #' \describe{
-## #'   \item{Y_samples}{Samples of the latent, Gaussian scale Y process}
-## #'   \item{mu_samples}{Samples of the conditional mean of the data}
-## #'   \item{prob_samples}{Samples of the probability of success parameter (only for the relevant response distributions)}
-## #'   \item{Z_samples}{Samples of the response variable}
-## #' }
-.MC_sampler <- function(M, X, type, n_MC, obs_fs, k, Q_L, obsidx, predict_BAUs, CP, kriging){
-  
-  ## Some matrices evaluated at observed BAUs only
-  X_O <- .constructX_O(M) 
-  S_O <- .constructS_O(M) 
-  
-  MC <- list()              
-  N   <- nrow(M@S0)   
-  mstar <- length(obsidx)
-  r   <- nbasis(M)   
-  
-  ## Number of fixed and random effects
-  p <- length(M@alphahat)
-  s <- r + mstar * M@include_fs
+simulate_xi <- function(object, nsim, type) {
   
   ## number of spatial and temporal BAUs
-  if (is(M@basis,"TensorP_Basis")) {
-    ns <- length(M@BAUs@sp)
-    nt <- length(unique(M@BAUs@endTime))
+  if (is(object@basis,"TensorP_Basis")) {
+    ns <- length(object@BAUs@sp)
+    nt <- length(unique(object@BAUs@endTime))
   } else {
-    ns <- length(M@BAUs)
+    ns <- length(object@BAUs)
   }
   
-  # ---- Generate samples from (eta', xi_O')' ----
+  N     <- nrow(object@S0)   
+  mstar <- length(observed_BAUs(object))
   
-  ## Must generate samples jointly, as elements of alpha, eta, and xi_O are correlated.
-  ## First, construct the mean vector containing all fixed effects 
+  if (type == "observed") {
+    f <- observed_BAUs 
+    nloc <- mstar
+  } else {
+    f <- unobserved_BAUs
+    nloc <- N - mstar
+  }
+  
+  
+  if (object@fs_by_spatial_BAU) {
+    idx <- f(object)
+    spatial_BAU_id <- ((idx - 1) %% ns) + 1
+    sigma2fs <- object@sigma2fshat[spatial_BAU_id]
+  } else {
+    sigma2fs <- object@sigma2fshat
+  }
+  
+  xi <- rnorm(nsim * nloc, mean = 0, sd = sqrt(sigma2fs)) %>% 
+    matrix(nrow = nloc, ncol = nsim)
+  
+  return(xi)
+}
+
+
+.simulate <- function(object, X, type, nsim, obs_fs, k, Q_L, predict_BAUs, CP, kriging, newdata){
+  
+  ## Design matrices evaluated at observed BAUs only
+  X_O <- .constructX_O(object) 
+  S_O <- .constructS_O(object) 
+  
+  obsidx <- observed_BAUs(object)        # index of observed BAUs
+  
+  MC <- list()              
+  N   <- nrow(object@S0)   
+  mstar <- length(obsidx)
+  r   <- nbasis(object)   
+  
+  ## Number of fixed and random effects
+  p <- length(object@alphahat)
+  s <- r + mstar * object@include_fs
+  
+  # ---- Define alpha and generate samples from (eta', xi_O')' ----
+  
+  ## Construct the posterior mean of all random effects (include the regression
+  ## coefficients, alpha, if kriging = "universal").
   ## (if kriging == "universal"), and random effects; the basis function random 
-  ## weights, and the fine-scale variation (if M@include_fs). 
-  mu_posterior <- if (kriging == "universal") as.numeric(M@alphahat) else vector()
-  mu_posterior <- c(mu_posterior, as.numeric(M@mu_eta))
-  if (M@include_fs) mu_posterior <- c(mu_posterior, as.numeric(M@mu_xi))
+  ## weights, and the fine-scale variation (if object@include_fs). 
+  posterior_mean <- if (kriging == "universal") as.numeric(object@alphahat) else vector()
+  posterior_mean <- c(posterior_mean, as.numeric(object@mu_eta))
+  if (object@include_fs) posterior_mean <- c(posterior_mean, as.numeric(object@mu_xi))
   
-  ## Now make a matrix with n_MC columns, whose columns are the mean vector repeated.
-  mu_posterior_Matrix  <- matrix(rep(mu_posterior, times = n_MC), ncol = n_MC)
-  
-  ## Finally, generate samples from Gau(0, 1) distribution, and transform this 
+  ## Generate samples from Gau(0, 1) distribution, and transform this 
   ## standard normal vector to one that has the desired posterior precision matrix.
-  z <- matrix(rnorm(length(mu_posterior) * n_MC), 
-              nrow = length(mu_posterior), ncol = n_MC)
+  z <- matrix(rnorm(length(posterior_mean) * nsim), 
+              nrow = length(posterior_mean), ncol = nsim)
   U <- Matrix::t(Q_L$Qpermchol) # upper Cholesky factor of permuted joint posterior precision matrix 
-  x <- solve(U, z)        # x ~ Gau(0, A), where A is the permuted precision matrix i.e. A = P'QP
-  y <- Q_L$P %*% x        # y ~ Gau(0, Q^{-1})
-  mu_posterior <- as.matrix(y + mu_posterior_Matrix) # add the mean to y
+  x <- solve(U, z)              # x ~ Gau(0, A), where A is the permuted precision matrix i.e. A = P'QP
+  y <- Q_L$P %*% x              # y ~ Gau(0, Q^-1)
+  y <- as.matrix(y)
   
-  ## Separate the MC samples of each quantity
+  ## Add the mean vector to the simulated random effects.
+  samples <- y + matrix(rep(posterior_mean, times = nsim), ncol = nsim) 
+  
+  ## Separate the MC samples of each quantity 
   if (kriging == "universal") {
-    alpha <- mu_posterior[1:p, , drop = FALSE]
-    eta   <- mu_posterior[(p + 1):(p + r), ]
-    if (M@include_fs)
-      xi_O  <- mu_posterior[(p + r + 1):(p + r + mstar), ]
-  } else if (kriging == "simple") {
-    eta   <- mu_posterior[1:r, ]
-    if (M@include_fs)
-      xi_O  <- mu_posterior[(r + 1):(r + mstar), ]
+    alpha <- samples[1:p, , drop = FALSE]
+    eta   <- samples[(p + 1):(p + r), ]
+  } else {
+    eta <- samples[1:r, ]
+  }
+  
+  ## Observed fine-scale variation:
+  if (object@include_fs) {
+    
+    if (kriging == "universal") {
+      xi_O  <- samples[(p + r + 1):(p + r + mstar), ]
+    } else {
+      xi_O  <- samples[(r + 1):(r + mstar), ]
+    }
+
   }
   
   ## We now have several matrices of Monte Carlo samples:
   ## alpha (if kriging = "universal"), eta, and xi_O (if include_fs = TRUE).
-  ## row i of eta corresponds to n_MC MC samples of eta_i,
-  ## row i of xi_O corresponds to n_MC MC samples of the fine-scale variation at the ith observed location.
+  ## row i of eta corresponds to nsim MC samples of eta_i,
+  ## row i of xi_O corresponds to nsim MC samples of the fine-scale variation at the ith observed location.
   
   # ---- Generate samples from xi_U ----
   
   ## This is straightforward as each element of xi_U is independent of
   ## all other random effects in the model.
-  ## All we have to do is make an (N-m) x n_MC matrix of draws from the
+  ## All we have to do is make an (N-m) x nsim matrix of draws from the
   ## Gaussian distribution with mean zero and variance equal to the fine-scale variance.
-  if (M@include_fs) {
-    
-    if (M@fs_by_spatial_BAU) {
-      unobsidx <- unobserved_BAUs(M)
-      spatial_BAU_id <- ((unobsidx - 1) %% ns) + 1
-      sigma2fs_U <- M@sigma2fshat[spatial_BAU_id]
-    } else {
-      sigma2fs_U <- M@sigma2fshat
-    }
-    
-    xi_U <- rnorm((N - mstar) * n_MC, mean = 0, sd = sqrt(sigma2fs_U)) %>% 
-      matrix(nrow = N - mstar, ncol = n_MC)
-    
+  if (object@include_fs) {
+    xi_U <- simulate_xi(object, nsim = nsim, type = "unobserved")
     xi_samples <- rbind(xi_O, xi_U)
   }
   
-
   # ---- Construct samples from the latent process Y ----
   
   ## We break the latent process down as: Y = Y_smooth + xi, 
@@ -633,28 +610,28 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   
   ## Split the covariate design matrix based on observed and unobserved samples
   X_U <- X[-obsidx, ]    # Unobserved fixed effect 'design' matrix
-  S_U <- M@S0[-obsidx, ] # Unobserved random effect 'design' matrix
+  S_U <- object@S0[-obsidx, ] # Unobserved random effect 'design' matrix
   
   ## Simulate the smooth Y-process (excluding fs variation) over the observed and unobserved BAUs
   if (kriging == "universal") {
     Y_smooth_O <- X_O %*% alpha + S_O %*% eta
     Y_smooth_U <- X_U %*% alpha + S_U %*% eta
   } else {
-    Y_smooth_O <- X_O %*% M@alphahat + S_O %*% eta
-    Y_smooth_U <- X_U %*% M@alphahat + S_U %*% eta
+    Y_smooth_O <- X_O %*% object@alphahat + S_O %*% eta
+    Y_smooth_U <- X_U %*% object@alphahat + S_U %*% eta
   }
   
   ## Combine samples
   Y_smooth_samples  <- rbind(Y_smooth_O, Y_smooth_U)
   
   ## Use permutation matrix to get the correct (original) ordering in terms of the BAUs
-  unobsidx         <- unobserved_BAUs(M)   # Unobserved BAUs indices
+  unobsidx         <- unobserved_BAUs(object)   # Unobserved BAUs indices
   ids              <- c(obsidx, unobsidx)  # All indices (observed and unobserved)
   P                <- Matrix::sparseMatrix(i = 1:N, j = 1:N, x = 1)[ids, ]
   Y_smooth_samples <- Matrix::t(P) %*% Y_smooth_samples
   
   ## Construct the samples from the latent process Y 
-  if (M@include_fs) {
+  if (object@include_fs) {
     xi_samples  <- Matrix::t(P) %*% xi_samples
     Y_samples   <- Y_smooth_samples + xi_samples
   } else {
@@ -667,14 +644,66 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   ## Outputted Y value depend on obs_fs
   MC$Y_samples <- if (obs_fs) Y_smooth_samples else Y_samples
   
-  ## If Y is the ONLY quantity of interest, exit the function.
+  ## If Y is the only quantity of interest, exit the function.
   if (!("mean" %in% type) & !("response" %in% type)) return(MC) 
   
   
   # ---- Apply inverse-link function to the samples to obtain conditional mean ----
+
+  ## For all models other than the Gau-Gau model (i.e., Gaussian data with an 
+  ## identity link function), the fine-scale variation process xi(.) is 
+  ## henceforth included in the latent process Y(.).
   
-  ## Past this point we must have xi in the Y process (the model breaks down otherwise).
-  ## In the case of type == "all", we simply export Y_smooth_samples as the samples of Y.
+  if (object@response == "gaussian" && object@link == "identity" && obs_fs) {
+    mu_samples   <- Y_smooth_samples
+  } else {
+    tmp <- .inverselink(Y_samples, object, k = k)
+    mu_samples   <- tmp$mu_samples
+    prob_samples <- tmp$prob_samples
+  }
+  
+  # ---- Predicting over arbitrary polygons ----
+  
+  if (!predict_BAUs) mu_samples <- as.matrix(CP %*% mu_samples)
+  
+  if (!predict_BAUs & object@response %in% c("binomial", "negative-binomial")) {
+    k_P <- CP %*% k
+    h   <- .link_fn("mu_to_prob", response = object@response)
+    prob_samples <- as.matrix(h(mu_samples, k_P))
+  }
+  
+  ## Output the mean samples. If probability parameter was computed, and 
+  ## we are predicting over the BAUs, also output.
+  MC$mu_samples <- mu_samples
+  if (!is.null(prob_samples)) MC$prob_samples <- prob_samples
+  
+  ## If the response is not a quantity of interest, exit the function
+  if (!("response" %in% type)) return(MC)
+  
+  
+  # ---- Sample the response variable, Z ----
+  
+  if (object@response == "gaussian") {
+    sigma2e <- .measurementerrorvariance(object, newdata)
+    if (object@link == "identity" && obs_fs) {
+      
+      ## Add the fine-scale variation to mu(.) and possibly aggregate over 
+      ## the prediction regions. Note that the mu_samples return in the list MC 
+      ## is still the smooth version without fine-scale variation; we just add 
+      ## it here so that it can be implicitly incorporated in the simulations of 
+      ## the response variable. 
+      mu_samples  <- Y_smooth_samples + xi_samples
+      if (!predict_BAUs) mu_samples <- as.matrix(CP %*% mu_samples)
+    }
+  } 
+  
+  MC$Z_samples <- .sampleZ(mu_samples = mu_samples, object = object, obs_fs = obs_fs, sigma2e = sigma2e, k = k)
+  
+  return(MC)
+}
+
+
+.inverselink <- function(Y_samples, object, k) {
   
   ## For families with a known constant parameter (binomial, negative-binomial),
   ## finv() maps the Gaussian scale Y process to the probability parameter p.
@@ -686,85 +715,56 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
   ## Note that for all cases other than type == "link", we need to compute the conditional mean samples.
   
   ## Create the relevant link functions.
-  if (M@response %in% c("binomial", "negative-binomial") & M@link %in% c("logit", "probit", "cloglog")) {
-    finv  <- .link_fn("Y_to_prob", link = M@link)
+  if (object@response %in% c("binomial", "negative-binomial") & object@link %in% c("logit", "probit", "cloglog")) {
+    finv  <- .link_fn("Y_to_prob", link = object@link)
   } else {
-    ginv  <- .link_fn("Y_to_mu", link = M@link) 
+    ginv  <- .link_fn("Y_to_mu", link = object@link) 
   }
   
-  if(M@response %in% c("binomial", "negative-binomial")) {
-    hinv  <- .link_fn("prob_to_mu", response = M@response)
-    h     <- .link_fn("mu_to_prob", response = M@response)
+  if(object@response %in% c("binomial", "negative-binomial")) {
+    hinv  <- .link_fn("prob_to_mu", response = object@response)
+    h     <- .link_fn("mu_to_prob", response = object@response)
   }
   
   ## Create the mu samples (and prob parameter if applicable)
-  if (M@response %in% c("binomial", "negative-binomial") & M@link %in% c("logit", "probit", "cloglog")) {
+  if (object@response %in% c("binomial", "negative-binomial") & object@link %in% c("logit", "probit", "cloglog")) {
     prob_samples <- finv(Y = Y_samples)
     mu_samples   <- hinv(p = prob_samples, k = k)
-  } else if (M@response == "negative-binomial" & M@link %in% c("log", "sqrt")) {
+  } else if (object@response == "negative-binomial" & object@link %in% c("log", "sqrt")) {
     mu_samples   <- k * ginv(Y_samples) 
     prob_samples <- h(mu = mu_samples, k = k)
-  } else if (M@response == "gaussian" && M@link == "identity" && obs_fs) {
-    mu_samples <- Y_smooth_samples
   } else {
-    mu_samples <- ginv(Y_samples)
+    mu_samples   <- ginv(Y_samples)
   }
   
+  if (!exists("prob_samples")) prob_samples <- NULL
   
-  # ---- Predicting over arbitrary polygons ----
+  return(list(mu_samples = mu_samples, prob_samples = prob_samples))
+}
+
+
+
+.sampleZ <- function(mu_samples, object, obs_fs, sigma2e, k) {
   
-  if (!predict_BAUs) mu_samples <- as.matrix(CP %*% mu_samples)
+  n    <- length(mu_samples)
+  nsim <- ncol(mu_samples)
   
-  if (!predict_BAUs & M@response %in% c("binomial", "negative-binomial")) {
-    k_P <- CP %*% k
-    prob_samples <- as.matrix(h(mu_samples, k_P))
-  }
-  
-  ## Output the mean samples. If probability parameter was computed, and 
-  ## we are predicting over the BAUs, also output.
-  MC$mu_samples <- mu_samples
-  if (exists("prob_samples")) MC$prob_samples <- prob_samples
-  
-  ## If the response is not a quanitity of interest, exit the function
-  if (!("response" %in% type)) return(MC)
-  
-  
-  # ---- Sample the response variable, Z ----
-  
-  n <- nrow(CP) * n_MC
-  
-  if (M@response == "poisson") {
+  if (object@response == "poisson") {
     Z_samples <- rpois(n, lambda = c(t(mu_samples)))
-  } else if (M@response == "gaussian") {
-    # measurement error variance
-    if (!.zero_range(diag(M@Ve))) {
-      sigma2e <- mean(diag(M@Ve))
-      cat("You have requested inference on the noisy data process. In a Gaussian setting, this requires the measurement error standard deviation; we are assuming it is spatially invariant, and is the average of std field supplied in the data.")
-    } else {
-      sigma2e <- M@Ve[1, 1]
-    }
-    
-    ## If obs_fs = TRUE, we need to add the fine-scale variance here, and 
-    ## possibly aggregate over prediction regions
-    if(M@link == "identity" && obs_fs) {
-      mu_samples <- Y_samples # This is the smooth process Y (equivalent to mu, because link = identity) + fine-scale variation
-      if (!predict_BAUs) 
-        mu_samples <- as.matrix(CP %*% mu_samples)
-    }
-    
+  } else if (object@response == "gaussian") { 
     Z_samples <- rnorm(n, mean = c(t(mu_samples)), sd = sqrt(sigma2e))
-  } else if (M@response == "gamma") {
+  } else if (object@response == "gamma") {
     theta <- 1 / c(t(mu_samples))    # canonical parameter
-    a <- 1/M@phi                     # shape parameter
+    a     <- 1/object@phi            # shape parameter
     beta  <- theta * a               # rate parameter (1/scale)
     Z_samples <- rgamma(n, shape = a, rate = beta)
-  } else if (M@response == "inverse-gaussian") {
-    Z_samples <- statmod::rinvgauss(n, mean = c(t(mu_samples)), dispersion = M@phi) 
-  } else if (M@response == "negative-binomial") {
-    k_vec <- rep(k, each = n_MC)
+  } else if (object@response == "inverse-gaussian") {
+    Z_samples <- statmod::rinvgauss(n, mean = c(t(mu_samples)), dispersion = object@phi) 
+  } else if (object@response == "negative-binomial") {
+    k_vec <- rep(k, each = nsim)
     Z_samples <- rnbinom(n, size = k_vec, mu = c(t(mu_samples)))
-  } else if (M@response == "binomial") {
-    k_vec <- rep(k, each = n_MC)
+  } else if (object@response == "binomial") {
+    k_vec <- rep(k, each = nsim)
     theta <- log((c(t(mu_samples))/k_vec) / (1 - (c(t(mu_samples))/k_vec)))
     p <- 1 / (1 + exp(-theta))
     ## NAs will occur if k = 0. Fortunately, if k = 0, we know Z will be 0. 
@@ -773,132 +773,28 @@ setMethod("predict", signature="SRE", function(object, newdata = NULL, obs_fs =
     Z_samples <- rbinom(n, size = k_vec, prob = p)
   }
   
-  ## Convert from a long vector to an n_pred_locs x n_MC matrix
-  Z_samples <- matrix(Z_samples, ncol = n_MC, byrow = TRUE)
-  
-  ## Add Z_samples to list object
-  MC$Z_samples <- Z_samples
-  
-  return(MC)
+  ## Convert from a long vector to an n_pred_locs x nsim matrix
+  Z_samples <- matrix(Z_samples, ncol = nsim, byrow = TRUE)
 }
 
-# ---- Analytic solution: Not implemented yet ----
-
-## Posterior variance of the latent Y process.
-## Computes the variance of the latent process \eqn{Y} at every BAU. 
-## Note that MSPE(E(Y|Z), Y) is approximated by var(Y|Z).
-## 
-## To compute the prediction uncertainty of Y we require the joint
-## covariance matrix of the random effects \eqn{(\eta', \xi_O')'}. \code{TMB}
-## provides an approximation of the joint \emph{precision} matrix of
-## \eqn{(\eta', \xi_O')'}, which we must invert to obtain the approximate
-## covariance matrix. However, due to the potentially very large number of
-## random effects (the number of observations \eqn{m} is not restricted),
-## in practice we compute the sparse-inverse-subset of the joint precision matrix,
-## \emph{not} the true joint covariance matrix.
-## 
-## Note that as we are using E(\eqn{Y|Z}) to predict \eqn{Y}, 
-## the posterior variance acts as an approximation of the mean-squared 
-## prediction error (see pg. 72 of Honours thesis).
-## 
-## #'@param M An object of class SRE.
-## #'@param Q_L A list containing the Cholesky factor of the permuted precision matrix (stored as \code{Q$Qpermchol}) and the associated permutationmatrix (stored as \code{Q_L$P}).
-## #'@param Q_posterior the posterior precision matrix of the fixed and random effects
-## #'@param obsidx Vector containing the observed locations.
-## #'@param X matrix of covariates
-## #'@param kriging whether we wish to perform "simple" or "universal" kriging
-## #'@return A vector of the posterior variance of Y at every BAU. 
-# .Y_var <- function(M, Q_posterior, Q_L, obsidx, X, kriging){
-#   
-#   r <- nbasis(M)
-#   mstar <- length(obsidx)
-#   
-#   ## Number of fixed and random effects
-#   p <- length(M@alphahat)
-#   s <- r + mstar * M@include_fs
-#   
-#   ## number of spatial and temporal BAUs
-#   if (is(M@basis,"TensorP_Basis")) {
-#     ns <- length(M@BAUs@sp)
-#     nt <- length(unique(M@BAUs@endTime))
-#   } else {
-#     ns <- length(M@BAUs)
-#   }
-#   
-#   
-#   # ---- Inverse of Q  
-#   
-#   ## Use the sparse-inverse-subset (acting as a proxy for the true covariance matrix)
-#   ## if we have too many random effects
-#   if (r + mstar < 4000) {
-#     Sigma <- chol2inv(chol(M@Q_posterior))
-#   } else {
-#     Sigma <- sparseinv::Takahashi_Davis(Q = Q_posterior, cholQp = Q_L$Qpermchol, P = Q_L$P)
-#   }
-#   
-#   if (kriging == "universal") {
-#     Sigma_alpha   <- Sigma[1:p, 1:p, drop = FALSE]
-#     Sigma_random  <- Sigma[-(1:p), -(1:p), drop = FALSE]
-#     Cov_alpha_eta <- Sigma[1:p, (p+1):(p+r), drop = FALSE]
-#     Cov_alpha_xi  <- Sigma[1:p, (p + r +1):(p+r+mstar), drop = FALSE]
-#   } else if (kriging == "simple") {
-#     Sigma_random  <- Sigma
-#   }
-#   
-#   Sigma_eta <- Sigma_random[1:r, 1:r]
-#   if (M@include_fs) {
-#     Sigma_xi    <- Sigma_random[(r + 1):(r + mstar), (r + 1):(r + mstar)]
-#     Cov_eta_xi  <- Sigma_random[1:r, (r + 1):(r + mstar)] # Covariances between xi_O and eta
-#   }
-#   
-#   
-#   # ----- Uncertainty: Posterior variance of Y at each BAU
-#   
-#   ## To extract the variances of eta|Z, we need diag(S0 %*% Sigma_eta %*% t(S0)).
-#   ## Also, to extract the covariance terms, we need: diag(S %*% COV_{eta, xi}).
-#   ## This in very inefficient to do directly, it much better to use the identity:
-#   ##      diag(AB) = (A*B')1
-#   
-#   
-#   ## Add common terms for both observed and unobserved locations:
-#   vY <- as.vector( (M@S0 %*% Sigma_eta * M@S0) %*% rep(1, r) )
-#   
-#   if(kriging == "universal") {
-#     
-#     ## Variance due to alpha
-#     vY <- vY + as.vector( (X %*% Sigma_alpha * X) %*% rep(1, p) )
-#     
-#     ## Covariance terms between alpha and eta
-#     cov_alpha_eta <- as.vector( (X %*% Cov_alpha_eta * M@S0) %*% rep(1, r) )
-#     vY <- vY + 2 * cov_alpha_eta
-#   }
-#   
-#   
-#   if (M@include_fs) {
-#     
-#     ## UNOBSERVED locations
-#     
-#     ## simply add the estimate of sigma2fs to the variance.
-#     ## If we have a unique fine-scale variance at each spatial BAU (spatio-temporal 
-#     ## case only), add the sigma2fs associated with that BAU.
-#     if (M@fs_by_spatial_BAU) {
-#       unobsidx <- unobserved_BAUs(M)
-#       spatial_BAU_id <- ((unobsidx - 1) %% ns) + 1
-#       vY[unobsidx] <- vY[unobsidx] + M@sigma2fshat[spatial_BAU_id]
-#     } else {
-#       vY[-obsidx] <- vY[-obsidx] + M@sigma2fshat
-#     }
-#     
-#     ## OBSERVED locations
-#     
-#     ## add both var(xi_O|Z) and cov(xi_O, eta | Z)
-#     vY[obsidx] <- vY[obsidx] + diag(Sigma_xi) + 2 * (M@S_O * t(Cov_eta_xi)) %*% rep(1, r)
-#     
-#     ## Add covariance between alpha and xi_O if we are using universal kriging
-#     if(kriging == "universal") 
-#       vY[obsidx] <- vY[obsidx] + 2 * (M@X_O * t(Cov_alpha_xi)) %*% rep(1, p)
-#     
-#   }
-#   
-#   return(vY) # Return variance of Y
-# }
+.measurementerrorvariance <- function(object, newdata) {
+  
+  if (!is.null(newdata$std)) {
+    sigma2e <- (newdata$std)^2
+  } else {
+    
+    # Ideally we would use all of Ve. However, Ve is of dimension m*, where m* 
+    # is the number of observed BAUs (rather than m, the number of 
+    # observations in newdata pre-binning). For now, just extract a single value 
+    # from Ve. 
+    
+    if (!.zero_range(diag(object@Ve))) {
+      sigma2e <- mean(diag(object@Ve))
+      cat("Prediction of Gaussian-distributed data requires the measurement-error standard deviation; we are assuming that it is spatially invariant, and that it is the average of measurement-error standard deviation used during model fitting (which were supplied with the data in the field 'std', or estimated using variogram techniques).")
+    } else {
+      sigma2e <- object@Ve[1, 1]
+    }
+  }
+  
+  return(sigma2e)
+}
